"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const child_process_1 = require("child_process");
const Stream = require("stream");
const util_1 = require("util");
const crypto_1 = require("crypto");
const path = require("path");
const os_1 = require("os");
const index_1 = require("./index");
const pipeline = util_1.promisify(Stream.pipeline);
describe('zip store stream', () => {
    it('should create valid empty ZIP archive', async () => {
        const FILENAME = 'test-empty.zip';
        const zip = new index_1.ZipStoreStream([]);
        await pipeline(zip, fs_1.createWriteStream(FILENAME));
        expect(() => child_process_1.execFileSync('unzip', ['-t', FILENAME], { encoding: 'utf8' })).toThrowError(expect.objectContaining({
            status: 1,
            stdout: expect.stringContaining(`warning [${FILENAME}]:  zipfile is empty`),
        }));
        fs_1.unlinkSync(FILENAME);
    });
    it('should create valid ZIP archive with ASCII and Unicode text files', async () => {
        const FILENAME = 'test-text.zip';
        const ASCII_TEXT = 'This is test of ASCII text';
        const UNICODE_TEXT = 'Я ❤️ JavaScript';
        const zip = new index_1.ZipStoreStream([
            { path: 'ascii.txt', data: ASCII_TEXT },
            { path: 'emoji.txt', data: UNICODE_TEXT },
        ]);
        await pipeline(zip, fs_1.createWriteStream(FILENAME));
        const ascii = child_process_1.execFileSync('unzip', ['-p', FILENAME, 'ascii.txt'], {
            encoding: 'utf-8',
        });
        expect(ascii).toBe(ASCII_TEXT);
        const unicode = child_process_1.execFileSync('unzip', ['-p', FILENAME, 'emoji.txt'], {
            encoding: 'utf-8',
        });
        expect(unicode).toBe(UNICODE_TEXT);
        fs_1.unlinkSync(FILENAME);
    });
    it('should work with binary buffers', async () => {
        const BUFF1_DATA = crypto_1.randomBytes(1024);
        const BUFF1_FILENAME = 'test/buffer1.bin';
        const BUFF2_DATA = crypto_1.randomBytes(4096);
        const BUFF2_FILENAME = 'test2/buf2.bin';
        const FILENAME = 'test-buffers.zip';
        const zip = new index_1.ZipStoreStream([
            { path: BUFF1_FILENAME, data: BUFF1_DATA },
            { path: BUFF2_FILENAME, data: BUFF2_DATA },
        ]);
        await pipeline(zip, fs_1.createWriteStream(FILENAME));
        const buf1 = child_process_1.execFileSync('unzip', ['-p', FILENAME, BUFF1_FILENAME], {
            encoding: 'buffer',
        });
        const buf2 = child_process_1.execFileSync('unzip', ['-p', FILENAME, BUFF2_FILENAME], {
            encoding: 'buffer',
        });
        expect(Buffer.compare(buf1, BUFF1_DATA)).toBe(0);
        expect(Buffer.compare(buf2, BUFF2_DATA)).toBe(0);
        fs_1.unlinkSync(FILENAME);
    });
    it('should work with streams', async () => {
        const FILENAME = 'test-stream.zip';
        const THIS_FILE = path.basename(__filename);
        const zip = new index_1.ZipStoreStream([
            {
                path: THIS_FILE,
                data: fs_1.createReadStream(__filename),
            },
        ]);
        await pipeline(zip, fs_1.createWriteStream(FILENAME));
        const content = child_process_1.execFileSync('unzip', ['-p', FILENAME, THIS_FILE], {
            encoding: 'utf-8',
        });
        expect(content).toBe(fs_1.readFileSync(__filename, { encoding: 'utf-8' }));
        fs_1.unlinkSync(FILENAME);
    });
    it('should work with mixed content', async () => {
        var _a;
        const FILENAME = 'test-mixed.zip';
        const zip = new index_1.ZipStoreStream([
            { path: 'string.txt', data: 'Hello world!' },
            { path: 'buffer.bin', data: crypto_1.randomBytes(10000) },
            {
                path: 'stream.ts',
                data: fs_1.createReadStream(__filename, { encoding: 'utf8' }),
            },
        ]);
        await pipeline(zip, fs_1.createWriteStream(FILENAME));
        expect(child_process_1.execFileSync('unzip', ['-t', FILENAME], { encoding: 'utf8' }))
            .toMatchInlineSnapshot(`
      "Archive:  test-mixed.zip
          testing: string.txt               OK
          testing: buffer.bin               OK
          testing: stream.ts                OK
      No errors detected in compressed data of test-mixed.zip.
      "
    `);
        const verbosely = child_process_1.execFileSync('unzip', ['-v', FILENAME], {
            encoding: 'utf8',
        });
        console.log(verbosely);
        /*
                Length   Method    Size  Cmpr    Date    Time   CRC-32   Name
            --------  ------  ------- ---- ---------- ----- --------  ----
                  12  Stored       12   0% 00-00-1980 00:00 1b851995  string.txt
              10000  Stored    10000   0% 00-00-1980 00:00 983c6c5a  buffer.bin
                3472  Stored     3472   0% 00-00-1980 00:00 f21cdbc5  stream.ts
            --------          -------  ---                            -------
              13484            13484   0%                            3 files
        */
        const parsed = verbosely
            .trim()
            .split(os_1.EOL)
            .slice(3, 6)
            .map(l => { var _a; return (_a = /^\s*(?<Length>\d+)\s+(?<Method>\w+)\s+(?<Size>\d+)\s+(?<Cmpr>\d+)%\s+(?<Date>[\d./-]{10,10})\s+(?<Time>\d\d:\d\d)\s+(?<CRC32>\S+)\s+(?<name>\S+)$/.exec(l)) === null || _a === void 0 ? void 0 : _a.groups; });
        expect(parsed).toHaveLength(3);
        expect(parsed).toEqual(expect.arrayContaining([
            expect.objectContaining({ Cmpr: '0', Method: 'Stored' }),
        ]));
        expect((_a = parsed[1]) === null || _a === void 0 ? void 0 : _a.Size).toBe('10000');
        fs_1.unlinkSync(FILENAME);
    });
});
//# sourceMappingURL=index.test.js.map